@@ -1,6 +1,6 @@
 //! Common types for the [push notifications module][push].
 //!
-//! [push]: https://matrix.org/docs/spec/client_server/r0.6.1#id89
+//! [push]: https://spec.matrix.org/v1.2/client-server-api/#push-notifications
 //!
 //! ## Understanding the types of this module
 //!
@@ -19,6 +19,10 @@ use std::hash::{Hash, Hasher};
 use indexmap::{Equivalent, IndexSet};
 use ruma_serde::{Raw, StringEnum};
 use serde::{Deserialize, Serialize};
+#[cfg(feature = "unstable-pre-spec")]
+use serde_json::Value as JsonValue;
+
+use crate::PrivOwnedStr;
 
 mod action;
 mod condition;
@@ -99,10 +103,7 @@ impl Ruleset {
         &self,
         event: &Raw<T>,
         context: &PushConditionRoomCtx,
-    ) -> Option<AnyPushRuleRef<'_>>
-    where
-        T: Serialize,
-    {
+    ) -> Option<AnyPushRuleRef<'_>> {
         let event = FlattenedJson::from_raw(event);
         self.iter().find(|rule| rule.applies(&event, context))
     }
@@ -115,10 +116,7 @@ impl Ruleset {
     ///
     /// * `event` - The raw JSON of a room message event.
     /// * `context` - The context of the message and room at the time of the event.
-    pub fn get_actions<T>(&self, event: &Raw<T>, context: &PushConditionRoomCtx) -> &[Action]
-    where
-        T: Serialize,
-    {
+    pub fn get_actions<T>(&self, event: &Raw<T>, context: &PushConditionRoomCtx) -> &[Action] {
         self.get_match(event, context).map(|rule| rule.actions()).unwrap_or(&[])
     }
 }
@@ -392,13 +390,26 @@ impl Equivalent<PatternedPushRule> for str {
 #[derive(Clone, Debug, Default, Serialize, Deserialize)]
 #[cfg_attr(not(feature = "unstable-exhaustive-types"), non_exhaustive)]
 pub struct PusherData {
-    /// Required if the pusher's kind is http. The URL to use to send notifications to.
+    /// The URL to use to send notifications to.
+    ///
+    /// Required if the pusher's kind is http.
     #[serde(skip_serializing_if = "Option::is_none")]
     pub url: Option<String>,
 
     /// The format to use when sending notifications to the Push Gateway.
     #[serde(skip_serializing_if = "Option::is_none")]
     pub format: Option<PushFormat>,
+
+    /// iOS (+ macOS?) specific default payload that will be sent to apple push notification
+    /// service.
+    ///
+    /// For more information, see [Sygnal docs][sygnal].
+    ///
+    /// [sygnal]: https://github.com/matrix-org/sygnal/blob/main/docs/applications.md#ios-applications-beware
+    // Not specified, issue: https://github.com/matrix-org/matrix-doc/issues/3474
+    #[cfg(feature = "unstable-pre-spec")]
+    #[serde(default, skip_serializing_if = "JsonValue::is_null")]
+    pub default_payload: JsonValue,
 }
 
 impl PusherData {
@@ -406,6 +417,19 @@ impl PusherData {
     pub fn new() -> Self {
         Default::default()
     }
+
+    /// Returns `true` if all fields are `None`.
+    pub fn is_empty(&self) -> bool {
+        #[cfg(not(feature = "unstable-pre-spec"))]
+        {
+            self.url.is_none() && self.format.is_none()
+        }
+
+        #[cfg(feature = "unstable-pre-spec")]
+        {
+            self.url.is_none() && self.format.is_none() && self.default_payload.is_null()
+        }
+    }
 }
 
 /// A special format that the homeserver should use when sending notifications to a Push Gateway.
@@ -414,7 +438,7 @@ impl PusherData {
 /// This type can hold an arbitrary string. To check for formats that are not available as a
 /// documented variant here, use its string representation, obtained through `.as_str()`.
 ///
-/// [spec]: https://matrix.org/docs/spec/push_gateway/r0.1.1#homeserver-behaviour
+/// [spec]: https://spec.matrix.org/v1.2/push-gateway-api/#homeserver-behaviour
 #[derive(Clone, Debug, PartialEq, Eq, StringEnum)]
 #[ruma_enum(rename_all = "snake_case")]
 #[non_exhaustive]
@@ -423,7 +447,7 @@ pub enum PushFormat {
     EventIdOnly,
 
     #[doc(hidden)]
-    _Custom(String),
+    _Custom(PrivOwnedStr),
 }
 
 impl PushFormat {
@@ -446,13 +470,12 @@ mod tests {
         value::RawValue as RawJsonValue, Value as JsonValue,
     };
 
-    use crate::power_levels::NotificationPowerLevels;
-
     use super::{
         action::{Action, Tweak},
         condition::{PushCondition, PushConditionRoomCtx, RoomMemberCountIs},
         AnyPushRule, ConditionalPushRule, PatternedPushRule, Ruleset, SimplePushRule,
     };
+    use crate::power_levels::NotificationPowerLevels;
 
     fn example_ruleset() -> Ruleset {
         let mut set = Ruleset::new();
@@ -936,10 +959,10 @@ mod tests {
 
     #[test]
     fn default_ruleset_applies() {
-        let set = Ruleset::server_default(&user_id!("@jolly_jumper:server.name"));
+        let set = Ruleset::server_default(user_id!("@jolly_jumper:server.name"));
 
         let context_one_to_one = &PushConditionRoomCtx {
-            room_id: room_id!("!dm:server.name"),
+            room_id: room_id!("!dm:server.name").to_owned(),
             member_count: 2_u32.into(),
             user_display_name: "Jolly Jumper".into(),
             users_power_levels: BTreeMap::new(),
@@ -948,7 +971,7 @@ mod tests {
         };
 
         let context_public_room = &PushConditionRoomCtx {
-            room_id: room_id!("!far_west:server.name"),
+            room_id: room_id!("!far_west:server.name").to_owned(),
             member_count: 100_u32.into(),
             user_display_name: "Jolly Jumper".into(),
             users_power_levels: BTreeMap::new(),
@@ -1038,7 +1061,7 @@ mod tests {
     #[test]
     fn custom_ruleset_applies() {
         let context_one_to_one = &PushConditionRoomCtx {
-            room_id: room_id!("!dm:server.name"),
+            room_id: room_id!("!dm:server.name").to_owned(),
             member_count: 2_u32.into(),
             user_display_name: "Jolly Jumper".into(),
             users_power_levels: BTreeMap::new(),
