@@ -19,7 +19,7 @@ pub use room_member_count_is::{ComparisonOperator, RoomMemberCountIs};
 #[cfg_attr(not(feature = "unstable-exhaustive-types"), non_exhaustive)]
 #[serde(tag = "kind", rename_all = "snake_case")]
 pub enum PushCondition {
-    /// This is a glob pattern match on a field of the event.
+    /// A glob pattern match on a field of the event.
     EventMatch {
         /// The dot-separated field of the event to match.
         key: String,
@@ -31,18 +31,18 @@ pub enum PushCondition {
         pattern: String,
     },
 
-    /// This matches unencrypted messages where `content.body` contains the owner's display name in
-    /// that room.
+    /// Matches unencrypted messages where `content.body` contains the owner's display name in that
+    /// room.
     ContainsDisplayName,
 
-    /// This matches the current number of members in the room.
+    /// Matches the current number of members in the room.
     RoomMemberCount {
         /// The condition on the current number of members in the room.
         is: RoomMemberCountIs,
     },
 
-    /// This takes into account the current power levels in the room, ensuring the sender of the
-    /// event has high enough power to trigger the notification.
+    /// Takes into account the current power levels in the room, ensuring the sender of the event
+    /// has high enough power to trigger the notification.
     SenderNotificationPermission {
         /// The field in the power level event the user needs a minimum power level for.
         ///
@@ -90,7 +90,7 @@ impl PushCondition {
             Self::RoomMemberCount { is } => is.contains(&context.member_count),
             Self::SenderNotificationPermission { key } => {
                 let sender_id = match event.get("sender") {
-                    Some(v) => match UserId::try_from(v) {
+                    Some(v) => match <&UserId>::try_from(v) {
                         Ok(u) => u,
                         Err(_) => return false,
                     },
@@ -99,7 +99,7 @@ impl PushCondition {
 
                 let sender_level = context
                     .users_power_levels
-                    .get(&sender_id)
+                    .get(sender_id)
                     .unwrap_or(&context.default_power_level);
 
                 match context.notification_power_levels.get(key) {
@@ -116,7 +116,7 @@ impl PushCondition {
 #[allow(clippy::exhaustive_structs)]
 pub struct PushConditionRoomCtx {
     /// The ID of the room.
-    pub room_id: RoomId,
+    pub room_id: Box<RoomId>,
 
     /// The number of members in the room.
     pub member_count: UInt,
@@ -125,7 +125,7 @@ pub struct PushConditionRoomCtx {
     pub user_display_name: String,
 
     /// The power levels of the users of the room.
-    pub users_power_levels: BTreeMap<UserId, Int>,
+    pub users_power_levels: BTreeMap<Box<UserId>, Int>,
 
     /// The default power level of the users of the room.
     pub default_power_level: Int,
@@ -271,14 +271,9 @@ pub struct FlattenedJson {
 
 impl FlattenedJson {
     /// Create a `FlattenedJson` from `Raw`.
-    pub fn from_raw<T>(raw: &Raw<T>) -> Self
-    where
-        T: Serialize,
-    {
+    pub fn from_raw<T>(raw: &Raw<T>) -> Self {
         let mut s = Self { map: BTreeMap::new() };
-
         s.flatten_value(to_json_value(raw).unwrap(), "".into());
-
         s
     }
 
@@ -296,12 +291,7 @@ impl FlattenedJson {
                     warn!("Duplicate path in flattened JSON: {}", path);
                 }
             }
-            JsonValue::Number(_) | JsonValue::Bool(_) => {
-                if self.map.insert(path.clone(), value.to_string()).is_some() {
-                    warn!("Duplicate path in flattened JSON: {}", path);
-                }
-            }
-            JsonValue::Array(_) | JsonValue::Null => {}
+            JsonValue::Number(_) | JsonValue::Bool(_) | JsonValue::Array(_) | JsonValue::Null => {}
         }
     }
 
@@ -324,9 +314,8 @@ mod tests {
         from_value as from_json_value, json, to_value as to_json_value, Value as JsonValue,
     };
 
-    use crate::power_levels::NotificationPowerLevels;
-
     use super::{FlattenedJson, PushCondition, PushConditionRoomCtx, RoomMemberCountIs, StrExt};
+    use crate::power_levels::NotificationPowerLevels;
 
     #[test]
     fn serialize_event_match_condition() {
@@ -478,13 +467,13 @@ mod tests {
 
     #[test]
     fn conditions_apply_to_events() {
-        let first_sender = user_id!("@worthy_whale:server.name");
+        let first_sender = user_id!("@worthy_whale:server.name").to_owned();
 
         let mut users_power_levels = BTreeMap::new();
         users_power_levels.insert(first_sender, 25.into());
 
         let context = PushConditionRoomCtx {
-            room_id: room_id!("!room:server.name"),
+            room_id: room_id!("!room:server.name").to_owned(),
             member_count: 3_u8.into(),
             user_display_name: "Groovy Gorilla".into(),
             users_power_levels,
@@ -577,14 +566,7 @@ mod tests {
         .unwrap();
 
         let flattened = FlattenedJson::from_raw(&raw);
-        assert_eq!(
-            flattened.map,
-            btreemap! {
-                "string".into() => "Hello World".into(),
-                "number".into() => "10".into(),
-                "boolean".into() => "true".into(),
-            },
-        );
+        assert_eq!(flattened.map, btreemap! { "string".into() => "Hello World".into() });
     }
 
     #[test]
