@@ -1,6 +1,6 @@
 //! Common types for the [third party networks module][thirdparty].
 //!
-//! [thirdparty]: https://matrix.org/docs/spec/client_server/r0.6.1#id153
+//! [thirdparty]: https://spec.matrix.org/v1.2/client-server-api/#third-party-networks
 
 use std::collections::BTreeMap;
 
@@ -8,7 +8,7 @@ use ruma_identifiers::{RoomAliasId, UserId};
 use ruma_serde::StringEnum;
 use serde::{Deserialize, Serialize};
 
-use crate::MilliSecondsSinceUnixEpoch;
+use crate::{MilliSecondsSinceUnixEpoch, PrivOwnedStr};
 
 /// Metadata about a third party protocol.
 ///
@@ -25,8 +25,8 @@ pub struct Protocol {
 
     /// A content URI representing an icon for the third party protocol.
     ///
-    /// If you activate the `compat` feature, this field being absent in JSON will give you an
-    /// empty string here.
+    /// If you activate the `compat` feature, this field being absent in JSON will result in an
+    /// empty string here during deserialization.
     #[cfg_attr(feature = "compat", serde(default))]
     pub icon: String,
 
@@ -88,6 +88,8 @@ pub struct ProtocolInstance {
     pub network_id: String,
 
     /// A unique identifier across all instances.
+    ///
+    /// See [matrix-doc#3203](https://github.com/matrix-org/matrix-doc/issues/3203).
     #[cfg(feature = "unstable-pre-spec")]
     pub instance_id: String,
 }
@@ -109,6 +111,8 @@ pub struct ProtocolInstanceInit {
     pub network_id: String,
 
     /// A unique identifier across all instances.
+    ///
+    /// See [matrix-doc#3203](https://github.com/matrix-org/matrix-doc/issues/3203).
     #[cfg(feature = "unstable-pre-spec")]
     pub instance_id: String,
 }
@@ -173,7 +177,7 @@ impl From<FieldTypeInit> for FieldType {
 #[cfg_attr(not(feature = "unstable-exhaustive-types"), non_exhaustive)]
 pub struct Location {
     /// An alias for a matrix room.
-    pub alias: RoomAliasId,
+    pub alias: Box<RoomAliasId>,
 
     /// The protocol ID that the third party location is a part of.
     pub protocol: String,
@@ -184,7 +188,11 @@ pub struct Location {
 
 impl Location {
     /// Creates a new `Location` with the given alias, protocol and fields.
-    pub fn new(alias: RoomAliasId, protocol: String, fields: BTreeMap<String, String>) -> Self {
+    pub fn new(
+        alias: Box<RoomAliasId>,
+        protocol: String,
+        fields: BTreeMap<String, String>,
+    ) -> Self {
         Self { alias, protocol, fields }
     }
 }
@@ -194,7 +202,7 @@ impl Location {
 #[cfg_attr(not(feature = "unstable-exhaustive-types"), non_exhaustive)]
 pub struct User {
     /// A matrix user ID representing a third party user.
-    pub userid: UserId,
+    pub userid: Box<UserId>,
 
     /// The protocol ID that the third party user is a part of.
     pub protocol: String,
@@ -205,7 +213,7 @@ pub struct User {
 
 impl User {
     /// Creates a new `User` with the given userid, protocol and fields.
-    pub fn new(userid: UserId, protocol: String, fields: BTreeMap<String, String>) -> Self {
+    pub fn new(userid: Box<UserId>, protocol: String, fields: BTreeMap<String, String>) -> Self {
         Self { userid, protocol, fields }
     }
 }
@@ -225,7 +233,7 @@ pub enum Medium {
     Msisdn,
 
     #[doc(hidden)]
-    _Custom(String),
+    _Custom(PrivOwnedStr),
 }
 
 impl Medium {
