@@ -1,13 +1,13 @@
 //! Common types for the Send-To-Device Messaging
 //!
-//! [send-to-device]: https://matrix.org/docs/spec/client_server/r0.6.1#id70
+//! [send-to-device]: https://spec.matrix.org/v1.2/client-server-api/#send-to-device-messaging
 
 use std::{
     convert::TryFrom,
     fmt::{Display, Formatter, Result as FmtResult},
 };
 
-use ruma_identifiers::DeviceIdBox;
+use ruma_identifiers::DeviceId;
 use serde::{
     de::{self, Unexpected},
     Deserialize, Deserializer, Serialize, Serializer,
@@ -18,7 +18,7 @@ use serde::{
 #[allow(clippy::exhaustive_enums)]
 pub enum DeviceIdOrAllDevices {
     /// Represents a device Id for one of a user's devices.
-    DeviceId(DeviceIdBox),
+    DeviceId(Box<DeviceId>),
 
     /// Represents all devices for a user.
     AllDevices,
@@ -27,20 +27,21 @@ pub enum DeviceIdOrAllDevices {
 impl Display for DeviceIdOrAllDevices {
     fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
         match self {
-            DeviceIdOrAllDevices::DeviceId(device_id) => write!(f, "{}", device_id.to_string()),
+            DeviceIdOrAllDevices::DeviceId(device_id) => write!(f, "{}", device_id),
             DeviceIdOrAllDevices::AllDevices => write!(f, "*"),
         }
     }
 }
 
-impl From<DeviceIdBox> for DeviceIdOrAllDevices {
-    fn from(d: DeviceIdBox) -> Self {
+impl From<Box<DeviceId>> for DeviceIdOrAllDevices {
+    fn from(d: Box<DeviceId>) -> Self {
         DeviceIdOrAllDevices::DeviceId(d)
     }
 }
 
 impl TryFrom<&str> for DeviceIdOrAllDevices {
     type Error = &'static str;
+
     fn try_from(device_id_or_all_devices: &str) -> Result<Self, Self::Error> {
         if device_id_or_all_devices.is_empty() {
             Err("Device identifier cannot be empty")
